"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from jsonpath import JSONPath
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.types import OptionalNullable, UNSET
from typing import Any, Dict, List, Mapping, Optional, Union


class Downloadables(BaseSDK):
    def list(
        self,
        *,
        security: Union[
            models.CustomerPortalDownloadablesListSecurity,
            models.CustomerPortalDownloadablesListSecurityTypedDict,
        ],
        organization_id: OptionalNullable[
            Union[
                models.CustomerPortalDownloadablesListQueryParamOrganizationIDFilter,
                models.CustomerPortalDownloadablesListQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        benefit_id: OptionalNullable[
            Union[
                models.CustomerPortalDownloadablesListQueryParamBenefitIDFilter,
                models.CustomerPortalDownloadablesListQueryParamBenefitIDFilterTypedDict,
            ]
        ] = UNSET,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CustomerPortalDownloadablesListResponse]:
        r"""List Downloadables

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param organization_id: Filter by organization ID.
        :param benefit_id: Filter by benefit ID.
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalDownloadablesListRequest(
            organization_id=organization_id,
            benefit_id=benefit_id,
            page=page,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/v1/customer-portal/downloadables/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalDownloadablesListSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="customer_portal:downloadables:list",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.CustomerPortalDownloadablesListResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list(
                security=security,
                organization_id=organization_id,
                benefit_id=benefit_id,
                page=next_page,
                limit=limit,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CustomerPortalDownloadablesListResponse(
                result=utils.unmarshal_json(
                    http_res.text, models.ListResourceDownloadableRead
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_async(
        self,
        *,
        security: Union[
            models.CustomerPortalDownloadablesListSecurity,
            models.CustomerPortalDownloadablesListSecurityTypedDict,
        ],
        organization_id: OptionalNullable[
            Union[
                models.CustomerPortalDownloadablesListQueryParamOrganizationIDFilter,
                models.CustomerPortalDownloadablesListQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        benefit_id: OptionalNullable[
            Union[
                models.CustomerPortalDownloadablesListQueryParamBenefitIDFilter,
                models.CustomerPortalDownloadablesListQueryParamBenefitIDFilterTypedDict,
            ]
        ] = UNSET,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CustomerPortalDownloadablesListResponse]:
        r"""List Downloadables

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param organization_id: Filter by organization ID.
        :param benefit_id: Filter by benefit ID.
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalDownloadablesListRequest(
            organization_id=organization_id,
            benefit_id=benefit_id,
            page=page,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/customer-portal/downloadables/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalDownloadablesListSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="customer_portal:downloadables:list",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.CustomerPortalDownloadablesListResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list(
                security=security,
                organization_id=organization_id,
                benefit_id=benefit_id,
                page=next_page,
                limit=limit,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CustomerPortalDownloadablesListResponse(
                result=utils.unmarshal_json(
                    http_res.text, models.ListResourceDownloadableRead
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        token: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[Any]:
        r"""Get Downloadable

        :param token:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.CustomerPortalDownloadablesCustomerPortalDownloadablesGetRequest(
                token=token,
            )
        )

        req = self._build_request(
            method="GET",
            path="/v1/customer-portal/downloadables/{token}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=False,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="customer_portal:downloadables:customer_portal.downloadables.get",
                oauth2_scopes=[],
                security_source=None,
            ),
            request=req,
            error_status_codes=["400", "404", "410", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "302", "*"):
            return None
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, ["400", "404", "410", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        token: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[Any]:
        r"""Get Downloadable

        :param token:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = (
            models.CustomerPortalDownloadablesCustomerPortalDownloadablesGetRequest(
                token=token,
            )
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/customer-portal/downloadables/{token}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=False,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="customer_portal:downloadables:customer_portal.downloadables.get",
                oauth2_scopes=[],
                security_source=None,
            ),
            request=req,
            error_status_codes=["400", "404", "410", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "302", "*"):
            return None
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.HTTPValidationErrorData
            )
            raise models.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, ["400", "404", "410", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class BenefitGrantMeterCreditPropertiesTypedDict(TypedDict):
    last_credited_meter_id: str
    last_credited_units: int
    last_credited_at: str


class BenefitGrantMeterCreditProperties(BaseModel):
    last_credited_meter_id: str

    last_credited_units: int

    last_credited_at: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .attachedcustomfield import AttachedCustomField, AttachedCustomFieldTypedDict
from .checkoutdiscountfixedonceforeverduration import (
    CheckoutDiscountFixedOnceForeverDuration,
    CheckoutDiscountFixedOnceForeverDurationTypedDict,
)
from .checkoutdiscountfixedrepeatduration import (
    CheckoutDiscountFixedRepeatDuration,
    CheckoutDiscountFixedRepeatDurationTypedDict,
)
from .checkoutdiscountpercentageonceforeverduration import (
    CheckoutDiscountPercentageOnceForeverDuration,
    CheckoutDiscountPercentageOnceForeverDurationTypedDict,
)
from .checkoutdiscountpercentagerepeatduration import (
    CheckoutDiscountPercentageRepeatDuration,
    CheckoutDiscountPercentageRepeatDurationTypedDict,
)
from .checkoutproduct import CheckoutProduct, CheckoutProductTypedDict
from .checkoutstatus import CheckoutStatus
from .legacyrecurringproductprice import (
    LegacyRecurringProductPrice,
    LegacyRecurringProductPriceTypedDict,
)
from .paymentprocessor import PaymentProcessor
from .productprice import ProductPrice, ProductPriceTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


CheckoutCustomFieldDataTypedDict = TypeAliasType(
    "CheckoutCustomFieldDataTypedDict", Union[str, int, bool, datetime]
)


CheckoutCustomFieldData = TypeAliasType(
    "CheckoutCustomFieldData", Union[str, int, bool, datetime]
)


CheckoutMetadataTypedDict = TypeAliasType(
    "CheckoutMetadataTypedDict", Union[str, int, float, bool]
)


CheckoutMetadata = TypeAliasType("CheckoutMetadata", Union[str, int, float, bool])


CheckoutProductPriceTypedDict = TypeAliasType(
    "CheckoutProductPriceTypedDict",
    Union[LegacyRecurringProductPriceTypedDict, ProductPriceTypedDict],
)
r"""Price of the selected product."""


CheckoutProductPrice = TypeAliasType(
    "CheckoutProductPrice", Union[LegacyRecurringProductPrice, ProductPrice]
)
r"""Price of the selected product."""


CheckoutDiscountTypedDict = TypeAliasType(
    "CheckoutDiscountTypedDict",
    Union[
        CheckoutDiscountPercentageOnceForeverDurationTypedDict,
        CheckoutDiscountFixedOnceForeverDurationTypedDict,
        CheckoutDiscountPercentageRepeatDurationTypedDict,
        CheckoutDiscountFixedRepeatDurationTypedDict,
    ],
)


CheckoutDiscount = TypeAliasType(
    "CheckoutDiscount",
    Union[
        CheckoutDiscountPercentageOnceForeverDuration,
        CheckoutDiscountFixedOnceForeverDuration,
        CheckoutDiscountPercentageRepeatDuration,
        CheckoutDiscountFixedRepeatDuration,
    ],
)


CustomerMetadataTypedDict = TypeAliasType(
    "CustomerMetadataTypedDict", Union[str, int, bool]
)


CustomerMetadata = TypeAliasType("CustomerMetadata", Union[str, int, bool])


class CheckoutTypedDict(TypedDict):
    r"""Checkout session data retrieved using an access token."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    payment_processor: PaymentProcessor
    status: CheckoutStatus
    client_secret: str
    r"""Client secret used to update and complete the checkout session from the client."""
    url: str
    r"""URL where the customer can access the checkout session."""
    expires_at: datetime
    r"""Expiration date and time of the checkout session."""
    success_url: str
    r"""URL where the customer will be redirected after a successful payment."""
    embed_origin: Nullable[str]
    r"""When checkout is embedded, represents the Origin of the page embedding the checkout. Used as a security measure to send messages only to the embedding page."""
    amount: int
    r"""Amount in cents, before discounts and taxes."""
    discount_amount: int
    r"""Discount amount in cents."""
    net_amount: int
    r"""Amount in cents, after discounts but before taxes."""
    tax_amount: Nullable[int]
    r"""Sales tax amount in cents. If `null`, it means there is no enough information yet to calculate it."""
    total_amount: int
    r"""Amount in cents, after discounts and taxes."""
    currency: str
    r"""Currency code of the checkout session."""
    product_id: str
    r"""ID of the product to checkout."""
    product_price_id: str
    r"""ID of the product price to checkout."""
    discount_id: Nullable[str]
    r"""ID of the discount applied to the checkout."""
    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    is_discount_applicable: bool
    r"""Whether the discount is applicable to the checkout. Typically, free and custom prices are not discountable."""
    is_free_product_price: bool
    r"""Whether the product price is free, regardless of discounts."""
    is_payment_required: bool
    r"""Whether the checkout requires payment, e.g. in case of free products or discounts that cover the total amount."""
    is_payment_setup_required: bool
    r"""Whether the checkout requires setting up a payment method, regardless of the amount, e.g. subscriptions that have first free cycles."""
    is_payment_form_required: bool
    r"""Whether the checkout requires a payment form, whether because of a payment or payment method setup."""
    customer_id: Nullable[str]
    customer_name: Nullable[str]
    r"""Name of the customer."""
    customer_email: Nullable[str]
    r"""Email address of the customer."""
    customer_ip_address: Nullable[str]
    customer_billing_address: Nullable[AddressTypedDict]
    customer_tax_id: Nullable[str]
    payment_processor_metadata: Dict[str, str]
    metadata: Dict[str, CheckoutMetadataTypedDict]
    customer_external_id: Nullable[str]
    r"""ID of the customer in your system. If a matching customer exists on Polar, the resulting order will be linked to this customer. Otherwise, a new customer will be created with this external ID set."""
    products: List[CheckoutProductTypedDict]
    r"""List of products available to select."""
    product: CheckoutProductTypedDict
    r"""Product data for a checkout session."""
    product_price: CheckoutProductPriceTypedDict
    r"""Price of the selected product."""
    discount: Nullable[CheckoutDiscountTypedDict]
    subscription_id: Nullable[str]
    attached_custom_fields: List[AttachedCustomFieldTypedDict]
    customer_metadata: Dict[str, CustomerMetadataTypedDict]
    custom_field_data: NotRequired[
        Dict[str, Nullable[CheckoutCustomFieldDataTypedDict]]
    ]
    r"""Key-value object storing custom field values."""


class Checkout(BaseModel):
    r"""Checkout session data retrieved using an access token."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    payment_processor: PaymentProcessor

    status: CheckoutStatus

    client_secret: str
    r"""Client secret used to update and complete the checkout session from the client."""

    url: str
    r"""URL where the customer can access the checkout session."""

    expires_at: datetime
    r"""Expiration date and time of the checkout session."""

    success_url: str
    r"""URL where the customer will be redirected after a successful payment."""

    embed_origin: Nullable[str]
    r"""When checkout is embedded, represents the Origin of the page embedding the checkout. Used as a security measure to send messages only to the embedding page."""

    amount: int
    r"""Amount in cents, before discounts and taxes."""

    discount_amount: int
    r"""Discount amount in cents."""

    net_amount: int
    r"""Amount in cents, after discounts but before taxes."""

    tax_amount: Nullable[int]
    r"""Sales tax amount in cents. If `null`, it means there is no enough information yet to calculate it."""

    total_amount: int
    r"""Amount in cents, after discounts and taxes."""

    currency: str
    r"""Currency code of the checkout session."""

    product_id: str
    r"""ID of the product to checkout."""

    product_price_id: str
    r"""ID of the product price to checkout."""

    discount_id: Nullable[str]
    r"""ID of the discount applied to the checkout."""

    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    is_discount_applicable: bool
    r"""Whether the discount is applicable to the checkout. Typically, free and custom prices are not discountable."""

    is_free_product_price: bool
    r"""Whether the product price is free, regardless of discounts."""

    is_payment_required: bool
    r"""Whether the checkout requires payment, e.g. in case of free products or discounts that cover the total amount."""

    is_payment_setup_required: bool
    r"""Whether the checkout requires setting up a payment method, regardless of the amount, e.g. subscriptions that have first free cycles."""

    is_payment_form_required: bool
    r"""Whether the checkout requires a payment form, whether because of a payment or payment method setup."""

    customer_id: Nullable[str]

    customer_name: Nullable[str]
    r"""Name of the customer."""

    customer_email: Nullable[str]
    r"""Email address of the customer."""

    customer_ip_address: Nullable[str]

    customer_billing_address: Nullable[Address]

    customer_tax_id: Nullable[str]

    payment_processor_metadata: Dict[str, str]

    metadata: Dict[str, CheckoutMetadata]

    customer_external_id: Nullable[str]
    r"""ID of the customer in your system. If a matching customer exists on Polar, the resulting order will be linked to this customer. Otherwise, a new customer will be created with this external ID set."""

    products: List[CheckoutProduct]
    r"""List of products available to select."""

    product: CheckoutProduct
    r"""Product data for a checkout session."""

    product_price: CheckoutProductPrice
    r"""Price of the selected product."""

    discount: Nullable[CheckoutDiscount]

    subscription_id: Nullable[str]

    attached_custom_fields: List[AttachedCustomField]

    customer_metadata: Dict[str, CustomerMetadata]

    custom_field_data: Optional[Dict[str, Nullable[CheckoutCustomFieldData]]] = None
    r"""Key-value object storing custom field values."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["custom_field_data"]
        nullable_fields = [
            "modified_at",
            "embed_origin",
            "tax_amount",
            "discount_id",
            "customer_id",
            "customer_name",
            "customer_email",
            "customer_ip_address",
            "customer_billing_address",
            "customer_tax_id",
            "customer_external_id",
            "discount",
            "subscription_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerSubscriptionUpdateProductTypedDict(TypedDict):
    product_id: str
    r"""Update subscription to another product."""


class CustomerSubscriptionUpdateProduct(BaseModel):
    product_id: str
    r"""Update subscription to another product."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TaxIDFormat(str, Enum):
    r"""List of supported tax ID formats.

    Ref: https://docs.stripe.com/billing/customer/tax-ids#supported-tax-id
    """

    AD_NRT = "ad_nrt"
    AE_TRN = "ae_trn"
    AR_CUIT = "ar_cuit"
    AU_ABN = "au_abn"
    AU_ARN = "au_arn"
    BG_UIC = "bg_uic"
    BH_VAT = "bh_vat"
    BO_TIN = "bo_tin"
    BR_CNPJ = "br_cnpj"
    BR_CPF = "br_cpf"
    CA_BN = "ca_bn"
    CA_GST_HST = "ca_gst_hst"
    CA_PST_BC = "ca_pst_bc"
    CA_PST_MB = "ca_pst_mb"
    CA_PST_SK = "ca_pst_sk"
    CA_QST = "ca_qst"
    CH_UID = "ch_uid"
    CH_VAT = "ch_vat"
    CL_TIN = "cl_tin"
    CN_TIN = "cn_tin"
    CO_NIT = "co_nit"
    CR_TIN = "cr_tin"
    DE_STN = "de_stn"
    DO_RCN = "do_rcn"
    EC_RUC = "ec_ruc"
    EG_TIN = "eg_tin"
    ES_CIF = "es_cif"
    EU_OSS_VAT = "eu_oss_vat"
    EU_VAT = "eu_vat"
    GB_VAT = "gb_vat"
    GE_VAT = "ge_vat"
    HK_BR = "hk_br"
    HR_OIB = "hr_oib"
    HU_TIN = "hu_tin"
    ID_NPWP = "id_npwp"
    IL_VAT = "il_vat"
    IN_GST = "in_gst"
    IS_VAT = "is_vat"
    JP_CN = "jp_cn"
    JP_RN = "jp_rn"
    JP_TRN = "jp_trn"
    KE_PIN = "ke_pin"
    KR_BRN = "kr_brn"
    KZ_BIN = "kz_bin"
    LI_UID = "li_uid"
    MX_RFC = "mx_rfc"
    MY_FRP = "my_frp"
    MY_ITN = "my_itn"
    MY_SST = "my_sst"
    NG_TIN = "ng_tin"
    NO_VAT = "no_vat"
    NO_VOEC = "no_voec"
    NZ_GST = "nz_gst"
    OM_VAT = "om_vat"
    PE_RUC = "pe_ruc"
    PH_TIN = "ph_tin"
    RO_TIN = "ro_tin"
    RS_PIB = "rs_pib"
    RU_INN = "ru_inn"
    RU_KPP = "ru_kpp"
    SA_VAT = "sa_vat"
    SG_GST = "sg_gst"
    SG_UEN = "sg_uen"
    SI_TIN = "si_tin"
    SV_NIT = "sv_nit"
    TH_VAT = "th_vat"
    TR_TIN = "tr_tin"
    TW_VAT = "tw_vat"
    UA_VAT = "ua_vat"
    US_EIN = "us_ein"
    UY_RUC = "uy_ruc"
    VE_RIF = "ve_rif"
    VN_TIN = "vn_tin"
    ZA_VAT = "za_vat"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .discountduration import DiscountDuration
from .discounttype import DiscountType
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


DiscountFixedRepeatDurationCreateMetadataTypedDict = TypeAliasType(
    "DiscountFixedRepeatDurationCreateMetadataTypedDict", Union[str, int, float, bool]
)


DiscountFixedRepeatDurationCreateMetadata = TypeAliasType(
    "DiscountFixedRepeatDurationCreateMetadata", Union[str, int, float, bool]
)


class DiscountFixedRepeatDurationCreateTypedDict(TypedDict):
    r"""Schema to create a fixed amount discount that is applied on every invoice
    for a certain number of months.
    """

    duration: DiscountDuration
    duration_in_months: int
    r"""Number of months the discount should be applied.

    For this to work on yearly pricing, you should multiply this by 12.
    For example, to apply the discount for 2 years, set this to 24.
    """
    type: DiscountType
    amount: int
    r"""Fixed amount to discount from the invoice total."""
    name: str
    r"""Name of the discount. Will be displayed to the customer when the discount is applied."""
    currency: NotRequired[str]
    r"""The currency. Currently, only `usd` is supported."""
    metadata: NotRequired[Dict[str, DiscountFixedRepeatDurationCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    code: NotRequired[Nullable[str]]
    r"""Code customers can use to apply the discount during checkout. Must be between 3 and 256 characters long and contain only alphanumeric characters.If not provided, the discount can only be applied via the API."""
    starts_at: NotRequired[Nullable[datetime]]
    r"""Optional timestamp after which the discount is redeemable."""
    ends_at: NotRequired[Nullable[datetime]]
    r"""Optional timestamp after which the discount is no longer redeemable."""
    max_redemptions: NotRequired[Nullable[int]]
    r"""Optional maximum number of times the discount can be redeemed."""
    products: NotRequired[Nullable[List[str]]]
    organization_id: NotRequired[Nullable[str]]
    r"""The ID of the organization owning the discount. **Required unless you use an organization token.**"""


class DiscountFixedRepeatDurationCreate(BaseModel):
    r"""Schema to create a fixed amount discount that is applied on every invoice
    for a certain number of months.
    """

    duration: DiscountDuration

    duration_in_months: int
    r"""Number of months the discount should be applied.

    For this to work on yearly pricing, you should multiply this by 12.
    For example, to apply the discount for 2 years, set this to 24.
    """

    type: DiscountType

    amount: int
    r"""Fixed amount to discount from the invoice total."""

    name: str
    r"""Name of the discount. Will be displayed to the customer when the discount is applied."""

    currency: Optional[str] = "usd"
    r"""The currency. Currently, only `usd` is supported."""

    metadata: Optional[Dict[str, DiscountFixedRepeatDurationCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    code: OptionalNullable[str] = UNSET
    r"""Code customers can use to apply the discount during checkout. Must be between 3 and 256 characters long and contain only alphanumeric characters.If not provided, the discount can only be applied via the API."""

    starts_at: OptionalNullable[datetime] = UNSET
    r"""Optional timestamp after which the discount is redeemable."""

    ends_at: OptionalNullable[datetime] = UNSET
    r"""Optional timestamp after which the discount is no longer redeemable."""

    max_redemptions: OptionalNullable[int] = UNSET
    r"""Optional maximum number of times the discount can be redeemed."""

    products: OptionalNullable[List[str]] = UNSET

    organization_id: OptionalNullable[str] = UNSET
    r"""The ID of the organization owning the discount. **Required unless you use an organization token.**"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "currency",
            "metadata",
            "code",
            "starts_at",
            "ends_at",
            "max_redemptions",
            "products",
            "organization_id",
        ]
        nullable_fields = [
            "code",
            "starts_at",
            "ends_at",
            "max_redemptions",
            "products",
            "organization_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class TokenResponseTypedDict(TypedDict):
    access_token: str
    expires_in: int
    refresh_token: Nullable[str]
    scope: str
    id_token: str
    token_type: Literal["Bearer"]


class TokenResponse(BaseModel):
    access_token: str

    expires_in: int

    refresh_token: Nullable[str]

    scope: str

    id_token: str

    TOKEN_TYPE: Annotated[
        Annotated[Literal["Bearer"], AfterValidator(validate_const("Bearer"))],
        pydantic.Field(alias="token_type"),
    ] = "Bearer"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["refresh_token"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class BenefitMeterCreditSubscriberPropertiesTypedDict(TypedDict):
    r"""Properties available to subscribers for a benefit of type `meter_unit`."""

    units: int
    meter_id: str


class BenefitMeterCreditSubscriberProperties(BaseModel):
    r"""Properties available to subscribers for a benefit of type `meter_unit`."""

    units: int

    meter_id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EventNamesSortProperty(str, Enum):
    NAME = "name"
    MINUS_NAME = "-name"
    OCCURRENCES = "occurrences"
    MINUS_OCCURRENCES = "-occurrences"
    FIRST_SEEN = "first_seen"
    MINUS_FIRST_SEEN = "-first_seen"
    LAST_SEEN = "last_seen"
    MINUS_LAST_SEEN = "-last_seen"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentmethodcard import PaymentMethodCard, PaymentMethodCardTypedDict
from .paymentmethodgeneric import PaymentMethodGeneric, PaymentMethodGenericTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, SecurityMetadata
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class CustomerPortalCustomersAddPaymentMethodSecurityTypedDict(TypedDict):
    customer_session: str


class CustomerPortalCustomersAddPaymentMethodSecurity(BaseModel):
    customer_session: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


CustomerPortalCustomersAddPaymentMethodResponseCustomerPortalCustomersAddPaymentMethodTypedDict = TypeAliasType(
    "CustomerPortalCustomersAddPaymentMethodResponseCustomerPortalCustomersAddPaymentMethodTypedDict",
    Union[PaymentMethodGenericTypedDict, PaymentMethodCardTypedDict],
)
r"""Payment method created."""


CustomerPortalCustomersAddPaymentMethodResponseCustomerPortalCustomersAddPaymentMethod = TypeAliasType(
    "CustomerPortalCustomersAddPaymentMethodResponseCustomerPortalCustomersAddPaymentMethod",
    Union[PaymentMethodGeneric, PaymentMethodCard],
)
r"""Payment method created."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class RefundStatus(str, Enum):
    PENDING = "pending"
    SUCCEEDED = "succeeded"
    FAILED = "failed"
    CANCELED = "canceled"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptioncancel import SubscriptionCancel, SubscriptionCancelTypedDict
from .subscriptionupdateproduct import (
    SubscriptionUpdateProduct,
    SubscriptionUpdateProductTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


SubscriptionUpdateTypedDict = TypeAliasType(
    "SubscriptionUpdateTypedDict",
    Union[SubscriptionUpdateProductTypedDict, SubscriptionCancelTypedDict],
)


SubscriptionUpdate = TypeAliasType(
    "SubscriptionUpdate", Union[SubscriptionUpdateProduct, SubscriptionCancel]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .attachedcustomfieldcreate import (
    AttachedCustomFieldCreate,
    AttachedCustomFieldCreateTypedDict,
)
from .productpricecustomcreate import (
    ProductPriceCustomCreate,
    ProductPriceCustomCreateTypedDict,
)
from .productpricefixedcreate import (
    ProductPriceFixedCreate,
    ProductPriceFixedCreateTypedDict,
)
from .productpricefreecreate import (
    ProductPriceFreeCreate,
    ProductPriceFreeCreateTypedDict,
)
from .productpricemeteredunitcreate import (
    ProductPriceMeteredUnitCreate,
    ProductPriceMeteredUnitCreateTypedDict,
)
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ProductCreateMetadataTypedDict = TypeAliasType(
    "ProductCreateMetadataTypedDict", Union[str, int, float, bool]
)


ProductCreateMetadata = TypeAliasType(
    "ProductCreateMetadata", Union[str, int, float, bool]
)


ProductCreatePricesTypedDict = TypeAliasType(
    "ProductCreatePricesTypedDict",
    Union[
        ProductPriceFreeCreateTypedDict,
        ProductPriceFixedCreateTypedDict,
        ProductPriceCustomCreateTypedDict,
        ProductPriceMeteredUnitCreateTypedDict,
    ],
)


ProductCreatePrices = TypeAliasType(
    "ProductCreatePrices",
    Union[
        ProductPriceFreeCreate,
        ProductPriceFixedCreate,
        ProductPriceCustomCreate,
        ProductPriceMeteredUnitCreate,
    ],
)


class ProductCreateTypedDict(TypedDict):
    r"""Schema to create a product."""

    name: str
    r"""The name of the product."""
    recurring_interval: Nullable[SubscriptionRecurringInterval]
    r"""The recurring interval of the product. If `None`, the product is a one-time purchase."""
    prices: List[ProductCreatePricesTypedDict]
    r"""List of available prices for this product. It should contain at most one static price (fixed, custom or free), and any number of metered prices. Metered prices are not supported on one-time purchase products."""
    metadata: NotRequired[Dict[str, ProductCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    description: NotRequired[Nullable[str]]
    r"""The description of the product."""
    medias: NotRequired[Nullable[List[str]]]
    r"""List of file IDs. Each one must be on the same organization as the product, of type `product_media` and correctly uploaded."""
    attached_custom_fields: NotRequired[List[AttachedCustomFieldCreateTypedDict]]
    r"""List of custom fields to attach."""
    organization_id: NotRequired[Nullable[str]]
    r"""The ID of the organization owning the product. **Required unless you use an organization token.**"""


class ProductCreate(BaseModel):
    r"""Schema to create a product."""

    name: str
    r"""The name of the product."""

    recurring_interval: Nullable[SubscriptionRecurringInterval]
    r"""The recurring interval of the product. If `None`, the product is a one-time purchase."""

    prices: List[ProductCreatePrices]
    r"""List of available prices for this product. It should contain at most one static price (fixed, custom or free), and any number of metered prices. Metered prices are not supported on one-time purchase products."""

    metadata: Optional[Dict[str, ProductCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    description: OptionalNullable[str] = UNSET
    r"""The description of the product."""

    medias: OptionalNullable[List[str]] = UNSET
    r"""List of file IDs. Each one must be on the same organization as the product, of type `product_media` and correctly uploaded."""

    attached_custom_fields: Optional[List[AttachedCustomFieldCreate]] = None
    r"""List of custom fields to attach."""

    organization_id: OptionalNullable[str] = UNSET
    r"""The ID of the organization owning the product. **Required unless you use an organization token.**"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "description",
            "medias",
            "attached_custom_fields",
            "organization_id",
        ]
        nullable_fields = [
            "description",
            "recurring_interval",
            "medias",
            "organization_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class ProductPriceMeterTypedDict(TypedDict):
    r"""A meter associated to a metered price."""

    id: str
    r"""The ID of the object."""
    name: str
    r"""The name of the meter."""


class ProductPriceMeter(BaseModel):
    r"""A meter associated to a metered price."""

    id: str
    r"""The ID of the object."""

    name: str
    r"""The name of the meter."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filterclause import FilterClause, FilterClauseTypedDict
from .filterconjunction import FilterConjunction
from polar_sdk.types import BaseModel
from typing import List, Union
from typing_extensions import TypeAliasType, TypedDict


class FilterTypedDict(TypedDict):
    conjunction: FilterConjunction
    clauses: List[ClausesTypedDict]


class Filter(BaseModel):
    conjunction: FilterConjunction

    clauses: List[Clauses]


ClausesTypedDict = TypeAliasType(
    "ClausesTypedDict", Union["FilterTypedDict", FilterClauseTypedDict]
)


Clauses = TypeAliasType("Clauses", Union["Filter", FilterClause])



from typing import TYPE_CHECKING, Any, TypeVar, Union

from .._base import Ingestion

if TYPE_CHECKING:
    from polar_sdk import EventCreateCustomerMetadataTypedDict


class BaseStrategy:
    """
    Base structure of an event ingestion strategy.

    :param event_name: The name of the events that'll be reported by the strategy.
    :param ingestion: The ingestion client to use for sending events.
    :param metadata: Extra metadata to include with all events.
    """

    def __init__(
        self,
        event_name: str,
        ingestion: Ingestion,
        metadata: Union[dict[str, "EventCreateCustomerMetadataTypedDict"], None] = None,
    ) -> None:
        self.event_name = event_name
        self._ingestion = ingestion
        self._metadata = metadata or {}

    def ingest(self, external_customer_id: str, *args: Any, **kwargs: Any) -> None:
        """
        Send an event to the ingestion queue.

        :param external_customer_id: The external customer ID to associate with the event.
        """
        raise NotImplementedError()


S = TypeVar("S", bound=BaseStrategy)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .paymentmethodcard import PaymentMethodCard, PaymentMethodCardTypedDict
from .paymentmethodgeneric import PaymentMethodGeneric, PaymentMethodGenericTypedDict
from polar_sdk.types import BaseModel
from typing import List, Union
from typing_extensions import TypeAliasType, TypedDict


ItemsTypedDict = TypeAliasType(
    "ItemsTypedDict", Union[PaymentMethodGenericTypedDict, PaymentMethodCardTypedDict]
)


Items = TypeAliasType("Items", Union[PaymentMethodGeneric, PaymentMethodCard])


class ListResourceUnionPaymentMethodCardPaymentMethodGenericTypedDict(TypedDict):
    items: List[ItemsTypedDict]
    pagination: PaginationTypedDict


class ListResourceUnionPaymentMethodCardPaymentMethodGeneric(BaseModel):
    items: List[Items]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitmetercreditcreateproperties import (
    BenefitMeterCreditCreateProperties,
    BenefitMeterCreditCreatePropertiesTypedDict,
)
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


BenefitMeterCreditUpdateMetadataTypedDict = TypeAliasType(
    "BenefitMeterCreditUpdateMetadataTypedDict", Union[str, int, float, bool]
)


BenefitMeterCreditUpdateMetadata = TypeAliasType(
    "BenefitMeterCreditUpdateMetadata", Union[str, int, float, bool]
)


class BenefitMeterCreditUpdateTypedDict(TypedDict):
    metadata: NotRequired[Dict[str, BenefitMeterCreditUpdateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    description: NotRequired[Nullable[str]]
    r"""The description of the benefit. Will be displayed on products having this benefit."""
    type: Literal["meter_credit"]
    properties: NotRequired[Nullable[BenefitMeterCreditCreatePropertiesTypedDict]]


class BenefitMeterCreditUpdate(BaseModel):
    metadata: Optional[Dict[str, BenefitMeterCreditUpdateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    description: OptionalNullable[str] = UNSET
    r"""The description of the benefit. Will be displayed on products having this benefit."""

    TYPE: Annotated[
        Annotated[
            Literal["meter_credit"], AfterValidator(validate_const("meter_credit"))
        ],
        pydantic.Field(alias="type"),
    ] = "meter_credit"

    properties: OptionalNullable[BenefitMeterCreditCreateProperties] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["metadata", "description", "properties"]
        nullable_fields = ["description", "properties"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkout import Checkout, CheckoutTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceCheckoutTypedDict(TypedDict):
    items: List[CheckoutTypedDict]
    pagination: PaginationTypedDict


class ListResourceCheckout(BaseModel):
    items: List[Checkout]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerbenefitgrantcustomupdate import (
    CustomerBenefitGrantCustomUpdate,
    CustomerBenefitGrantCustomUpdateTypedDict,
)
from .customerbenefitgrantdiscordupdate import (
    CustomerBenefitGrantDiscordUpdate,
    CustomerBenefitGrantDiscordUpdateTypedDict,
)
from .customerbenefitgrantdownloadablesupdate import (
    CustomerBenefitGrantDownloadablesUpdate,
    CustomerBenefitGrantDownloadablesUpdateTypedDict,
)
from .customerbenefitgrantgithubrepositoryupdate import (
    CustomerBenefitGrantGitHubRepositoryUpdate,
    CustomerBenefitGrantGitHubRepositoryUpdateTypedDict,
)
from .customerbenefitgrantlicensekeysupdate import (
    CustomerBenefitGrantLicenseKeysUpdate,
    CustomerBenefitGrantLicenseKeysUpdateTypedDict,
)
from .customerbenefitgrantmetercreditupdate import (
    CustomerBenefitGrantMeterCreditUpdate,
    CustomerBenefitGrantMeterCreditUpdateTypedDict,
)
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


CustomerBenefitGrantUpdateTypedDict = TypeAliasType(
    "CustomerBenefitGrantUpdateTypedDict",
    Union[
        CustomerBenefitGrantDownloadablesUpdateTypedDict,
        CustomerBenefitGrantLicenseKeysUpdateTypedDict,
        CustomerBenefitGrantCustomUpdateTypedDict,
        CustomerBenefitGrantMeterCreditUpdateTypedDict,
        CustomerBenefitGrantDiscordUpdateTypedDict,
        CustomerBenefitGrantGitHubRepositoryUpdateTypedDict,
    ],
)


CustomerBenefitGrantUpdate = Annotated[
    Union[
        Annotated[CustomerBenefitGrantCustomUpdate, Tag("custom")],
        Annotated[CustomerBenefitGrantDiscordUpdate, Tag("discord")],
        Annotated[CustomerBenefitGrantDownloadablesUpdate, Tag("downloadables")],
        Annotated[CustomerBenefitGrantGitHubRepositoryUpdate, Tag("github_repository")],
        Annotated[CustomerBenefitGrantLicenseKeysUpdate, Tag("license_keys")],
        Annotated[CustomerBenefitGrantMeterCreditUpdate, Tag("meter_credit")],
    ],
    Discriminator(lambda m: get_discriminator(m, "benefit_type", "benefit_type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitpublic import BenefitPublic, BenefitPublicTypedDict
from .legacyrecurringproductprice import (
    LegacyRecurringProductPrice,
    LegacyRecurringProductPriceTypedDict,
)
from .productmediafileread import ProductMediaFileRead, ProductMediaFileReadTypedDict
from .productprice import ProductPrice, ProductPriceTypedDict
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Union
from typing_extensions import TypeAliasType, TypedDict


CustomerProductPricesTypedDict = TypeAliasType(
    "CustomerProductPricesTypedDict",
    Union[LegacyRecurringProductPriceTypedDict, ProductPriceTypedDict],
)


CustomerProductPrices = TypeAliasType(
    "CustomerProductPrices", Union[LegacyRecurringProductPrice, ProductPrice]
)


class CustomerProductTypedDict(TypedDict):
    r"""Schema of a product for customer portal."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the product."""
    name: str
    r"""The name of the product."""
    description: Nullable[str]
    r"""The description of the product."""
    recurring_interval: Nullable[SubscriptionRecurringInterval]
    r"""The recurring interval of the product. If `None`, the product is a one-time purchase."""
    is_recurring: bool
    r"""Whether the product is a subscription."""
    is_archived: bool
    r"""Whether the product is archived and no longer available."""
    organization_id: str
    r"""The ID of the organization owning the product."""
    prices: List[CustomerProductPricesTypedDict]
    r"""List of available prices for this product."""
    benefits: List[BenefitPublicTypedDict]
    r"""The benefits granted by the product."""
    medias: List[ProductMediaFileReadTypedDict]
    r"""The medias associated to the product."""


class CustomerProduct(BaseModel):
    r"""Schema of a product for customer portal."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the product."""

    name: str
    r"""The name of the product."""

    description: Nullable[str]
    r"""The description of the product."""

    recurring_interval: Nullable[SubscriptionRecurringInterval]
    r"""The recurring interval of the product. If `None`, the product is a one-time purchase."""

    is_recurring: bool
    r"""Whether the product is a subscription."""

    is_archived: bool
    r"""Whether the product is archived and no longer available."""

    organization_id: str
    r"""The ID of the organization owning the product."""

    prices: List[CustomerProductPrices]
    r"""List of available prices for this product."""

    benefits: List[BenefitPublic]
    r"""The benefits granted by the product."""

    medias: List[ProductMediaFileRead]
    r"""The medias associated to the product."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at", "description", "recurring_interval"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customfieldselectoption import (
    CustomFieldSelectOption,
    CustomFieldSelectOptionTypedDict,
)
from polar_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CustomFieldSelectPropertiesTypedDict(TypedDict):
    options: List[CustomFieldSelectOptionTypedDict]
    form_label: NotRequired[str]
    form_help_text: NotRequired[str]
    form_placeholder: NotRequired[str]


class CustomFieldSelectProperties(BaseModel):
    options: List[CustomFieldSelectOption]

    form_label: Optional[str] = None

    form_help_text: Optional[str] = None

    form_placeholder: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .s3filecreatemultipart import S3FileCreateMultipart, S3FileCreateMultipartTypedDict
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, NotRequired, TypedDict


class DownloadableFileCreateTypedDict(TypedDict):
    r"""Schema to create a file to be associated with the downloadables benefit."""

    name: str
    mime_type: str
    size: int
    upload: S3FileCreateMultipartTypedDict
    organization_id: NotRequired[Nullable[str]]
    checksum_sha256_base64: NotRequired[Nullable[str]]
    service: Literal["downloadable"]
    version: NotRequired[Nullable[str]]


class DownloadableFileCreate(BaseModel):
    r"""Schema to create a file to be associated with the downloadables benefit."""

    name: str

    mime_type: str

    size: int

    upload: S3FileCreateMultipart

    organization_id: OptionalNullable[str] = UNSET

    checksum_sha256_base64: OptionalNullable[str] = UNSET

    SERVICE: Annotated[
        Annotated[
            Literal["downloadable"], AfterValidator(validate_const("downloadable"))
        ],
        pydantic.Field(alias="service"),
    ] = "downloadable"

    version: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["organization_id", "checksum_sha256_base64", "version"]
        nullable_fields = ["organization_id", "checksum_sha256_base64", "version"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrganizationSocialPlatforms(str, Enum):
    X = "x"
    GITHUB = "github"
    FACEBOOK = "facebook"
    INSTAGRAM = "instagram"
    YOUTUBE = "youtube"
    TIKTOK = "tiktok"
    LINKEDIN = "linkedin"
    OTHER = "other"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class LegacyRecurringProductPriceCustomTypedDict(TypedDict):
    r"""A pay-what-you-want recurring price for a product, i.e. a subscription.

    **Deprecated**: The recurring interval should be set on the product itself.
    """

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the price."""
    is_archived: bool
    r"""Whether the price is archived and no longer available."""
    product_id: str
    r"""The ID of the product owning the price."""
    recurring_interval: SubscriptionRecurringInterval
    price_currency: str
    r"""The currency."""
    minimum_amount: Nullable[int]
    r"""The minimum amount the customer can pay."""
    maximum_amount: Nullable[int]
    r"""The maximum amount the customer can pay."""
    preset_amount: Nullable[int]
    r"""The initial amount shown to the customer."""
    amount_type: Literal["custom"]
    type: Literal["recurring"]
    r"""The type of the price."""
    legacy: Literal[True]


class LegacyRecurringProductPriceCustom(BaseModel):
    r"""A pay-what-you-want recurring price for a product, i.e. a subscription.

    **Deprecated**: The recurring interval should be set on the product itself.
    """

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the price."""

    is_archived: bool
    r"""Whether the price is archived and no longer available."""

    product_id: str
    r"""The ID of the product owning the price."""

    recurring_interval: SubscriptionRecurringInterval

    price_currency: str
    r"""The currency."""

    minimum_amount: Nullable[int]
    r"""The minimum amount the customer can pay."""

    maximum_amount: Nullable[int]
    r"""The maximum amount the customer can pay."""

    preset_amount: Nullable[int]
    r"""The initial amount shown to the customer."""

    AMOUNT_TYPE: Annotated[
        Annotated[Literal["custom"], AfterValidator(validate_const("custom"))],
        pydantic.Field(alias="amount_type"),
    ] = "custom"

    TYPE: Annotated[
        Annotated[Literal["recurring"], AfterValidator(validate_const("recurring"))],
        pydantic.Field(alias="type"),
    ] = "recurring"
    r"""The type of the price."""

    LEGACY: Annotated[
        Annotated[Literal[True], AfterValidator(validate_const(True))],
        pydantic.Field(alias="legacy"),
    ] = True

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "minimum_amount",
            "maximum_amount",
            "preset_amount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitgrantcustomproperties import (
    BenefitGrantCustomProperties,
    BenefitGrantCustomPropertiesTypedDict,
)
from .benefitgrantdiscordproperties import (
    BenefitGrantDiscordProperties,
    BenefitGrantDiscordPropertiesTypedDict,
)
from .benefitgrantdownloadablesproperties import (
    BenefitGrantDownloadablesProperties,
    BenefitGrantDownloadablesPropertiesTypedDict,
)
from .benefitgrantgithubrepositoryproperties import (
    BenefitGrantGitHubRepositoryProperties,
    BenefitGrantGitHubRepositoryPropertiesTypedDict,
)
from .benefitgrantlicensekeysproperties import (
    BenefitGrantLicenseKeysProperties,
    BenefitGrantLicenseKeysPropertiesTypedDict,
)
from .benefittype import BenefitType
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


CustomerStateBenefitGrantPropertiesTypedDict = TypeAliasType(
    "CustomerStateBenefitGrantPropertiesTypedDict",
    Union[
        BenefitGrantCustomPropertiesTypedDict,
        BenefitGrantDownloadablesPropertiesTypedDict,
        BenefitGrantLicenseKeysPropertiesTypedDict,
        BenefitGrantDiscordPropertiesTypedDict,
        BenefitGrantGitHubRepositoryPropertiesTypedDict,
    ],
)


CustomerStateBenefitGrantProperties = TypeAliasType(
    "CustomerStateBenefitGrantProperties",
    Union[
        BenefitGrantCustomProperties,
        BenefitGrantDownloadablesProperties,
        BenefitGrantLicenseKeysProperties,
        BenefitGrantDiscordProperties,
        BenefitGrantGitHubRepositoryProperties,
    ],
)


class CustomerStateBenefitGrantTypedDict(TypedDict):
    r"""An active benefit grant for a customer."""

    id: str
    r"""The ID of the grant."""
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    granted_at: datetime
    r"""The timestamp when the benefit was granted."""
    benefit_id: str
    r"""The ID of the benefit concerned by this grant."""
    benefit_type: BenefitType
    properties: CustomerStateBenefitGrantPropertiesTypedDict


class CustomerStateBenefitGrant(BaseModel):
    r"""An active benefit grant for a customer."""

    id: str
    r"""The ID of the grant."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    granted_at: datetime
    r"""The timestamp when the benefit was granted."""

    benefit_id: str
    r"""The ID of the benefit concerned by this grant."""

    benefit_type: BenefitType

    properties: CustomerStateBenefitGrantProperties

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class EventsIngestResponseTypedDict(TypedDict):
    inserted: int
    r"""Number of events inserted."""


class EventsIngestResponse(BaseModel):
    inserted: int
    r"""Number of events inserted."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BenefitGrantLicenseKeysPropertiesTypedDict(TypedDict):
    license_key_id: NotRequired[str]
    display_key: NotRequired[str]


class BenefitGrantLicenseKeysProperties(BaseModel):
    license_key_id: Optional[str] = None

    display_key: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerupdate import CustomerUpdate, CustomerUpdateTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CustomersUpdateExternalRequestTypedDict(TypedDict):
    external_id: str
    r"""The customer external ID."""
    customer_update: CustomerUpdateTypedDict


class CustomersUpdateExternalRequest(BaseModel):
    external_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The customer external ID."""

    customer_update: Annotated[
        CustomerUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CheckoutUpdatePublicCustomFieldDataTypedDict = TypeAliasType(
    "CheckoutUpdatePublicCustomFieldDataTypedDict", Union[str, int, bool, datetime]
)


CheckoutUpdatePublicCustomFieldData = TypeAliasType(
    "CheckoutUpdatePublicCustomFieldData", Union[str, int, bool, datetime]
)


class CheckoutUpdatePublicTypedDict(TypedDict):
    r"""Update an existing checkout session using the client secret."""

    custom_field_data: NotRequired[
        Dict[str, Nullable[CheckoutUpdatePublicCustomFieldDataTypedDict]]
    ]
    r"""Key-value object storing custom field values."""
    product_id: NotRequired[Nullable[str]]
    r"""ID of the product to checkout. Must be present in the checkout's product list."""
    product_price_id: NotRequired[Nullable[str]]
    r"""ID of the product price to checkout. Must correspond to a price present in the checkout's product list."""
    amount: NotRequired[Nullable[int]]
    customer_name: NotRequired[Nullable[str]]
    customer_email: NotRequired[Nullable[str]]
    customer_billing_address: NotRequired[Nullable[AddressTypedDict]]
    customer_tax_id: NotRequired[Nullable[str]]
    discount_code: NotRequired[Nullable[str]]
    r"""Discount code to apply to the checkout."""


class CheckoutUpdatePublic(BaseModel):
    r"""Update an existing checkout session using the client secret."""

    custom_field_data: Optional[
        Dict[str, Nullable[CheckoutUpdatePublicCustomFieldData]]
    ] = None
    r"""Key-value object storing custom field values."""

    product_id: OptionalNullable[str] = UNSET
    r"""ID of the product to checkout. Must be present in the checkout's product list."""

    product_price_id: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""ID of the product price to checkout. Must correspond to a price present in the checkout's product list."""

    amount: OptionalNullable[int] = UNSET

    customer_name: OptionalNullable[str] = UNSET

    customer_email: OptionalNullable[str] = UNSET

    customer_billing_address: OptionalNullable[Address] = UNSET

    customer_tax_id: OptionalNullable[str] = UNSET

    discount_code: OptionalNullable[str] = UNSET
    r"""Discount code to apply to the checkout."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "custom_field_data",
            "product_id",
            "product_price_id",
            "amount",
            "customer_name",
            "customer_email",
            "customer_billing_address",
            "customer_tax_id",
            "discount_code",
        ]
        nullable_fields = [
            "product_id",
            "product_price_id",
            "amount",
            "customer_name",
            "customer_email",
            "customer_billing_address",
            "customer_tax_id",
            "discount_code",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .userinfoorganization import UserInfoOrganization, UserInfoOrganizationTypedDict
from .userinfouser import UserInfoUser, UserInfoUserTypedDict
from typing import Union
from typing_extensions import TypeAliasType


Oauth2UserinfoResponseOauth2UserinfoTypedDict = TypeAliasType(
    "Oauth2UserinfoResponseOauth2UserinfoTypedDict",
    Union[UserInfoOrganizationTypedDict, UserInfoUserTypedDict],
)
r"""Successful Response"""


Oauth2UserinfoResponseOauth2Userinfo = TypeAliasType(
    "Oauth2UserinfoResponseOauth2Userinfo", Union[UserInfoOrganization, UserInfoUser]
)
r"""Successful Response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .meterquantity import MeterQuantity, MeterQuantityTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class MeterQuantitiesTypedDict(TypedDict):
    quantities: List[MeterQuantityTypedDict]
    total: float
    r"""The total quantity for the period."""


class MeterQuantities(BaseModel):
    quantities: List[MeterQuantity]

    total: float
    r"""The total quantity for the period."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkout import Checkout, CheckoutTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookCheckoutUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a checkout is updated.

    **Discord & Slack support:** Basic
    """

    data: CheckoutTypedDict
    r"""Checkout session data retrieved using an access token."""
    type: Literal["checkout.updated"]


class WebhookCheckoutUpdatedPayload(BaseModel):
    r"""Sent when a checkout is updated.

    **Discord & Slack support:** Basic
    """

    data: Checkout
    r"""Checkout session data retrieved using an access token."""

    TYPE: Annotated[
        Annotated[
            Literal["checkout.updated"],
            AfterValidator(validate_const("checkout.updated")),
        ],
        pydantic.Field(alias="type"),
    ] = "checkout.updated"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customermeter import CustomerMeter, CustomerMeterTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceCustomerMeterTypedDict(TypedDict):
    items: List[CustomerMeterTypedDict]
    pagination: PaginationTypedDict


class ListResourceCustomerMeter(BaseModel):
    items: List[CustomerMeter]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutsortproperty import CheckoutSortProperty
from .listresource_checkout_ import ListResourceCheckout, ListResourceCheckoutTypedDict
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CheckoutsListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "CheckoutsListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


CheckoutsListQueryParamOrganizationIDFilter = TypeAliasType(
    "CheckoutsListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


CheckoutsListQueryParamProductIDFilterTypedDict = TypeAliasType(
    "CheckoutsListQueryParamProductIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by product ID."""


CheckoutsListQueryParamProductIDFilter = TypeAliasType(
    "CheckoutsListQueryParamProductIDFilter", Union[str, List[str]]
)
r"""Filter by product ID."""


class CheckoutsListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[CheckoutsListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    product_id: NotRequired[Nullable[CheckoutsListQueryParamProductIDFilterTypedDict]]
    r"""Filter by product ID."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[CheckoutSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class CheckoutsListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[CheckoutsListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    product_id: Annotated[
        OptionalNullable[CheckoutsListQueryParamProductIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product ID."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[CheckoutSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["organization_id", "product_id", "page", "limit", "sorting"]
        nullable_fields = ["organization_id", "product_id", "sorting"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CheckoutsListResponseTypedDict(TypedDict):
    result: ListResourceCheckoutTypedDict


class CheckoutsListResponse(BaseModel):
    next: Callable[[], Optional[CheckoutsListResponse]]

    result: ListResourceCheckout



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitcustom import BenefitCustom, BenefitCustomTypedDict
from .benefitdiscord import BenefitDiscord, BenefitDiscordTypedDict
from .benefitdownloadables import BenefitDownloadables, BenefitDownloadablesTypedDict
from .benefitgithubrepository import (
    BenefitGitHubRepository,
    BenefitGitHubRepositoryTypedDict,
)
from .benefitlicensekeys import BenefitLicenseKeys, BenefitLicenseKeysTypedDict
from .benefitmetercredit import BenefitMeterCredit, BenefitMeterCreditTypedDict
from typing import Union
from typing_extensions import TypeAliasType


BenefitTypedDict = TypeAliasType(
    "BenefitTypedDict",
    Union[
        BenefitCustomTypedDict,
        BenefitDiscordTypedDict,
        BenefitGitHubRepositoryTypedDict,
        BenefitDownloadablesTypedDict,
        BenefitLicenseKeysTypedDict,
        BenefitMeterCreditTypedDict,
    ],
)


Benefit = TypeAliasType(
    "Benefit",
    Union[
        BenefitCustom,
        BenefitDiscord,
        BenefitGitHubRepository,
        BenefitDownloadables,
        BenefitLicenseKeys,
        BenefitMeterCredit,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerorderproduct import CustomerOrderProduct, CustomerOrderProductTypedDict
from .customerordersubscription import (
    CustomerOrderSubscription,
    CustomerOrderSubscriptionTypedDict,
)
from .orderitemschema import OrderItemSchema, OrderItemSchemaTypedDict
from .orderstatus import OrderStatus
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List
from typing_extensions import Annotated, TypedDict


class CustomerOrderTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    status: OrderStatus
    paid: bool
    r"""Whether the order has been paid for."""
    subtotal_amount: int
    r"""Amount in cents, before discounts and taxes."""
    discount_amount: int
    r"""Discount amount in cents."""
    net_amount: int
    r"""Amount in cents, after discounts but before taxes."""
    tax_amount: int
    r"""Sales tax amount in cents."""
    total_amount: int
    r"""Amount in cents, after discounts and taxes."""
    refunded_amount: int
    r"""Amount refunded in cents."""
    refunded_tax_amount: int
    r"""Sales tax refunded in cents."""
    currency: str
    customer_id: str
    product_id: str
    subscription_id: Nullable[str]
    user_id: str
    product: CustomerOrderProductTypedDict
    subscription: Nullable[CustomerOrderSubscriptionTypedDict]
    items: List[OrderItemSchemaTypedDict]
    r"""Line items composing the order."""


class CustomerOrder(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str

    status: OrderStatus

    paid: bool
    r"""Whether the order has been paid for."""

    subtotal_amount: int
    r"""Amount in cents, before discounts and taxes."""

    discount_amount: int
    r"""Discount amount in cents."""

    net_amount: int
    r"""Amount in cents, after discounts but before taxes."""

    tax_amount: int
    r"""Sales tax amount in cents."""

    total_amount: int
    r"""Amount in cents, after discounts and taxes."""

    refunded_amount: int
    r"""Amount refunded in cents."""

    refunded_tax_amount: int
    r"""Sales tax refunded in cents."""

    currency: str

    customer_id: str

    product_id: str

    subscription_id: Nullable[str]

    user_id: Annotated[
        str,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    product: CustomerOrderProduct

    subscription: Nullable[CustomerOrderSubscription]

    items: List[OrderItemSchema]
    r"""Line items composing the order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at", "subscription_id", "subscription"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrderStatus(str, Enum):
    PENDING = "pending"
    PAID = "paid"
    REFUNDED = "refunded"
    PARTIALLY_REFUNDED = "partially_refunded"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .refund import Refund, RefundTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookRefundCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a refund is created regardless of status.

    **Discord & Slack support:** Full
    """

    data: RefundTypedDict
    type: Literal["refund.created"]


class WebhookRefundCreatedPayload(BaseModel):
    r"""Sent when a refund is created regardless of status.

    **Discord & Slack support:** Full
    """

    data: Refund

    TYPE: Annotated[
        Annotated[
            Literal["refund.created"], AfterValidator(validate_const("refund.created"))
        ],
        pydantic.Field(alias="type"),
    ] = "refund.created"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customercancellationreason import CustomerCancellationReason
from .customersubscriptionmeter import (
    CustomerSubscriptionMeter,
    CustomerSubscriptionMeterTypedDict,
)
from .customersubscriptionproduct import (
    CustomerSubscriptionProduct,
    CustomerSubscriptionProductTypedDict,
)
from .legacyrecurringproductprice import (
    LegacyRecurringProductPrice,
    LegacyRecurringProductPriceTypedDict,
)
from .productprice import ProductPrice, ProductPriceTypedDict
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from .subscriptionstatus import SubscriptionStatus
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Union
from typing_extensions import TypeAliasType, TypedDict


CustomerSubscriptionPricesTypedDict = TypeAliasType(
    "CustomerSubscriptionPricesTypedDict",
    Union[LegacyRecurringProductPriceTypedDict, ProductPriceTypedDict],
)


CustomerSubscriptionPrices = TypeAliasType(
    "CustomerSubscriptionPrices", Union[LegacyRecurringProductPrice, ProductPrice]
)


class CustomerSubscriptionTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    amount: int
    r"""The amount of the subscription."""
    currency: str
    r"""The currency of the subscription."""
    recurring_interval: SubscriptionRecurringInterval
    status: SubscriptionStatus
    current_period_start: datetime
    r"""The start timestamp of the current billing period."""
    current_period_end: Nullable[datetime]
    r"""The end timestamp of the current billing period."""
    cancel_at_period_end: bool
    r"""Whether the subscription will be canceled at the end of the current period."""
    canceled_at: Nullable[datetime]
    r"""The timestamp when the subscription was canceled. The subscription might still be active if `cancel_at_period_end` is `true`."""
    started_at: Nullable[datetime]
    r"""The timestamp when the subscription started."""
    ends_at: Nullable[datetime]
    r"""The timestamp when the subscription will end."""
    ended_at: Nullable[datetime]
    r"""The timestamp when the subscription ended."""
    customer_id: str
    r"""The ID of the subscribed customer."""
    product_id: str
    r"""The ID of the subscribed product."""
    discount_id: Nullable[str]
    r"""The ID of the applied discount, if any."""
    checkout_id: Nullable[str]
    customer_cancellation_reason: Nullable[CustomerCancellationReason]
    customer_cancellation_comment: Nullable[str]
    product: CustomerSubscriptionProductTypedDict
    prices: List[CustomerSubscriptionPricesTypedDict]
    r"""List of enabled prices for the subscription."""
    meters: List[CustomerSubscriptionMeterTypedDict]
    r"""List of meters associated with the subscription."""


class CustomerSubscription(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    amount: int
    r"""The amount of the subscription."""

    currency: str
    r"""The currency of the subscription."""

    recurring_interval: SubscriptionRecurringInterval

    status: SubscriptionStatus

    current_period_start: datetime
    r"""The start timestamp of the current billing period."""

    current_period_end: Nullable[datetime]
    r"""The end timestamp of the current billing period."""

    cancel_at_period_end: bool
    r"""Whether the subscription will be canceled at the end of the current period."""

    canceled_at: Nullable[datetime]
    r"""The timestamp when the subscription was canceled. The subscription might still be active if `cancel_at_period_end` is `true`."""

    started_at: Nullable[datetime]
    r"""The timestamp when the subscription started."""

    ends_at: Nullable[datetime]
    r"""The timestamp when the subscription will end."""

    ended_at: Nullable[datetime]
    r"""The timestamp when the subscription ended."""

    customer_id: str
    r"""The ID of the subscribed customer."""

    product_id: str
    r"""The ID of the subscribed product."""

    discount_id: Nullable[str]
    r"""The ID of the applied discount, if any."""

    checkout_id: Nullable[str]

    customer_cancellation_reason: Nullable[CustomerCancellationReason]

    customer_cancellation_comment: Nullable[str]

    product: CustomerSubscriptionProduct

    prices: List[CustomerSubscriptionPrices]
    r"""List of enabled prices for the subscription."""

    meters: List[CustomerSubscriptionMeter]
    r"""List of meters associated with the subscription."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "current_period_end",
            "canceled_at",
            "started_at",
            "ends_at",
            "ended_at",
            "discount_id",
            "checkout_id",
            "customer_cancellation_reason",
            "customer_cancellation_comment",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



import atexit
import contextlib
import datetime
import logging
import queue
import threading
from collections.abc import Callable
from typing import TYPE_CHECKING, Union

from polar_sdk import Polar

if TYPE_CHECKING:
    from polar_sdk import EventCreateCustomerMetadataTypedDict, EventsModelTypedDict

    from .strategies._base import S

logger = logging.getLogger("polar_sdk.ingestion")


class Ingestion:
    """
    Event ingestion client for Polar.

    This class handles the ingestion of events into the Polar API without blocking
    the main thread, by using a background thread sending them in batches.

    :param access_token: The access_token required for authentication
    :param server: The server by name to use for all methods
    :param server_url: The server URL to use for all methods
    :param max_batch_size: The maximum number of events to send in a single batch.
    :param flush_interval: The interval in seconds to wait before flushing events.
    :param max_queue_size: The maximum number of events to keep in the queue.
    """

    def __init__(
        self,
        access_token: Union[str, Callable[[], Union[str, None]], None] = None,
        server: Union[str, None] = None,
        server_url: Union[str, None] = None,
        *,
        max_batch_size: int = 50,
        flush_interval: float = 5.0,
        max_queue_size: int = 10000,
    ) -> None:
        self.max_batch_size = max_batch_size
        self.flush_interval = flush_interval
        self.max_queue_size = max_queue_size

        self._stack = contextlib.ExitStack()
        self._client = self._stack.enter_context(
            Polar(access_token, server, server_url)
        )

        self._queue = queue.Queue["EventsModelTypedDict"](maxsize=max_queue_size)

        self._thread = threading.Thread(target=self._worker, daemon=True)
        self._should_close_thread = threading.Event()
        self._thread.start()

        atexit.register(self.close)

    def strategy(
        self,
        strategy_class: type["S"],
        event_name: str,
        **metadata: "EventCreateCustomerMetadataTypedDict",
    ) -> "S":
        """
        Instantiate a strategy tied to this ingestion client.

        :param strategy_class: The strategy class to instantiate.
        :param event_name: The name of the events that'll be reported by the strategy.
        :param metadata: Extra metadata to include with all events.
        """
        return strategy_class(event_name, self, metadata=metadata)

    def ingest(self, event: "EventsModelTypedDict") -> None:
        """
        Send an event to the ingestion queue.

        :param event: The event to send.
        """
        if event.get("timestamp") is None:
            event["timestamp"] = datetime.datetime.now(datetime.timezone.utc)
        self._queue.put(event, block=False)

    def flush(self, max_batch_size: Union[int, None] = None) -> None:
        """
        Flush events from the queue to the API

        :param max_batch_size: The maximum number of events to send in the batch.
        If `None`, there is no limit.
        """
        if self._queue.empty():
            return

        # Collect events up to max_batch_size
        events: list[EventsModelTypedDict] = []
        try:
            while not self._queue.empty() and (
                max_batch_size is None or len(events) < max_batch_size
            ):
                events.append(self._queue.get_nowait())
        except queue.Empty:
            pass

        if not events:
            return

        # Send the batch to the API
        try:
            self._send_batch(events)
            # Mark tasks as done only after successful sending
            for _ in events:
                self._queue.task_done()
        except Exception as e:
            # On failure, put events back in the queue for retry
            logger.error("Failed to send events: %s", e)
            for event in events:
                try:
                    self._queue.put(event, block=False)
                except queue.Full:
                    logger.error("Queue full, dropping event during retry")

    def close(self) -> None:
        """
        Flush remaining events and close the background thread.

        It's called automatically on program exit.
        """
        logger.debug("Shutting down, flushing remaining events...")
        self._should_close_thread.set()

        # Try to flush remaining events
        try:
            self.flush()
        except Exception as e:
            logger.error("Error during shutdown flush: %s", e)

        if self._thread.is_alive():
            self._thread.join(timeout=1.0)

        self._stack.close()
        logger.debug("Shutdown complete")

    def _worker(self) -> None:
        logger.debug("Worker thread started")
        while not self._should_close_thread.is_set():
            try:
                self.flush(self.max_batch_size)
            except Exception as e:
                logger.error("Error in worker thread: %s", e)

            self._should_close_thread.wait(timeout=self.flush_interval)

    def _send_batch(self, events: list["EventsModelTypedDict"]) -> None:
        response = self._client.events.ingest(request={"events": events})
        logger.debug("Ingested %d events", response.inserted)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customersubscriptionupdate import (
    CustomerSubscriptionUpdate,
    CustomerSubscriptionUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import (
    FieldMetadata,
    PathParamMetadata,
    RequestMetadata,
    SecurityMetadata,
)
from typing_extensions import Annotated, TypedDict


class CustomerPortalSubscriptionsUpdateSecurityTypedDict(TypedDict):
    customer_session: str


class CustomerPortalSubscriptionsUpdateSecurity(BaseModel):
    customer_session: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


class CustomerPortalSubscriptionsUpdateRequestTypedDict(TypedDict):
    id: str
    r"""The subscription ID."""
    customer_subscription_update: CustomerSubscriptionUpdateTypedDict


class CustomerPortalSubscriptionsUpdateRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The subscription ID."""

    customer_subscription_update: Annotated[
        CustomerSubscriptionUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk import utils
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated


class PaymentErrorData(BaseModel):
    detail: str

    ERROR: Annotated[
        Annotated[
            Literal["PaymentError"], AfterValidator(validate_const("PaymentError"))
        ],
        pydantic.Field(alias="error"),
    ] = "PaymentError"


class PaymentError(Exception):
    data: PaymentErrorData

    def __init__(self, data: PaymentErrorData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, PaymentErrorData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class LicenseKeyActivateConditionsTypedDict(TypedDict):
    pass


class LicenseKeyActivateConditions(BaseModel):
    pass


class LicenseKeyActivateMetaTypedDict(TypedDict):
    pass


class LicenseKeyActivateMeta(BaseModel):
    pass


class LicenseKeyActivateTypedDict(TypedDict):
    key: str
    organization_id: str
    label: str
    conditions: NotRequired[LicenseKeyActivateConditionsTypedDict]
    meta: NotRequired[LicenseKeyActivateMetaTypedDict]


class LicenseKeyActivate(BaseModel):
    key: str

    organization_id: str

    label: str

    conditions: Optional[LicenseKeyActivateConditions] = None

    meta: Optional[LicenseKeyActivateMeta] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EventSortProperty(str, Enum):
    TIMESTAMP = "timestamp"
    MINUS_TIMESTAMP = "-timestamp"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customersubscription import CustomerSubscription, CustomerSubscriptionTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceCustomerSubscriptionTypedDict(TypedDict):
    items: List[CustomerSubscriptionTypedDict]
    pagination: PaginationTypedDict


class ListResourceCustomerSubscription(BaseModel):
    items: List[CustomerSubscription]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitcustomupdate import BenefitCustomUpdate, BenefitCustomUpdateTypedDict
from .benefitdiscordupdate import BenefitDiscordUpdate, BenefitDiscordUpdateTypedDict
from .benefitdownloadablesupdate import (
    BenefitDownloadablesUpdate,
    BenefitDownloadablesUpdateTypedDict,
)
from .benefitgithubrepositoryupdate import (
    BenefitGitHubRepositoryUpdate,
    BenefitGitHubRepositoryUpdateTypedDict,
)
from .benefitlicensekeysupdate import (
    BenefitLicenseKeysUpdate,
    BenefitLicenseKeysUpdateTypedDict,
)
from .benefitmetercreditupdate import (
    BenefitMeterCreditUpdate,
    BenefitMeterCreditUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


BenefitsUpdateBenefitUpdateTypedDict = TypeAliasType(
    "BenefitsUpdateBenefitUpdateTypedDict",
    Union[
        BenefitCustomUpdateTypedDict,
        BenefitDiscordUpdateTypedDict,
        BenefitGitHubRepositoryUpdateTypedDict,
        BenefitDownloadablesUpdateTypedDict,
        BenefitLicenseKeysUpdateTypedDict,
        BenefitMeterCreditUpdateTypedDict,
    ],
)


BenefitsUpdateBenefitUpdate = TypeAliasType(
    "BenefitsUpdateBenefitUpdate",
    Union[
        BenefitCustomUpdate,
        BenefitDiscordUpdate,
        BenefitGitHubRepositoryUpdate,
        BenefitDownloadablesUpdate,
        BenefitLicenseKeysUpdate,
        BenefitMeterCreditUpdate,
    ],
)


class BenefitsUpdateRequestTypedDict(TypedDict):
    id: str
    request_body: BenefitsUpdateBenefitUpdateTypedDict


class BenefitsUpdateRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    request_body: Annotated[
        BenefitsUpdateBenefitUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
